/* Copyright 2022 NVIDIA Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#pragma once

// Useful for IDEs
#include "cunumeric/search/argwhere.h"
#include "cunumeric/pitches.h"

namespace cunumeric {

using namespace Legion;
using namespace legate;

template <VariantKind KIND, LegateTypeCode CODE, int DIM>
struct ArgWhereImplBody;

template <VariantKind KIND>
struct ArgWhereImpl {
  template <LegateTypeCode CODE, int DIM>
  void operator()(ArgWhereArgs& args) const
  {
    using VAL = legate_type_of<CODE>;

    auto rect_in = args.in.shape<DIM>();

    Pitches<DIM - 1> pitches;
    size_t volume = pitches.flatten(rect_in);

    if (volume == 0) {
      auto extents = Point<2>::ZEROES();
      // auto extents = Point<2>(0,DIM);
      auto buffer = create_buffer<VAL, 2>(extents);
      args.out.return_data(buffer, extents);
      return;
    }

    auto in = args.in.read_accessor<VAL, DIM>(rect_in);
    ArgWhereImplBody<KIND, CODE, DIM>()(args.out, in, pitches, rect_in, volume);
  }
};

template <VariantKind KIND>
static void argwhere_template(TaskContext& context)
{
  ArgWhereArgs args{context.outputs()[0], context.inputs()[0]};
  double_dispatch(args.in.dim(), args.in.code(), ArgWhereImpl<KIND>{}, args);
}

}  // namespace cunumeric
